using System;
using NUnit.Extensions.Converter.Evaluators;

namespace NUnit.Extensions.Converter
{
	public class BeanValueConverter : EvaluationConverter
	{
		private string format;
		private Type type;
		private string[] properties;

		public BeanValueConverter(Converter converter, Type type, params string[] properties)
			: this(converter, null, type, properties)
		{
		}
		public BeanValueConverter(Converter converter, string format, Type type, params string[] properties)
			: base(converter)
		{
			this.format = format;
			this.type = type;
			this.properties = properties;
			AddEvaluator(new BeanEvaluator());
		}
		protected override bool CanConvertSelf(object o)
		{
			return type.IsInstanceOfType(o);
		}
		protected override object ConvertSelf(object o)
		{
			if (properties.Length == 0) {
				properties = Beans.GetPropertyNames(o);
			}
			SetEvaluationKeys(properties);
			object result = base.ConvertSelf(o);
			if (format != null) {
				Formatter formatter = new Formatter(outermostConverter);
				return formatter.Format(format, result);
			}
			return result;
		}
	}
}
