using System;
using System.Collections;
using System.Data.SqlClient;

namespace NUnit.Extensions.Converter.Sql
{
	public class SqlSelect 
	{
		public string Text;
		public SqlConnection Connection;
		public SqlTransaction Transaction;
		public string RowFormat;
		public SqlSelect(string text, SqlConnection connection)
			: this(text, null, connection)
		{
		}
		public SqlSelect(string text, string rowFormat, SqlConnection connection)
			: this(text, rowFormat, connection, null)
		{
		}
		public SqlSelect(string text, SqlConnection connection, SqlTransaction transaction)
			: this(text, null, connection, transaction)
		{
		}
		public SqlSelect(string text, string rowFormat, SqlConnection connection, SqlTransaction transaction)
		{
			this.Text = text;
			this.RowFormat = rowFormat;
			this.Connection = connection;
			this.Transaction = transaction;
		}
	}

	public class SqlSelectConverter : SpecificObjectConverter 
	{
		public SqlSelectConverter(Converter converter) : base(converter)
		{
		}
		protected override bool CanConvertSelf(object o)
		{
			return o is SqlSelect;
		}
		protected override object ConvertSelf(object o)
		{
			SqlSelect select = o as SqlSelect;
			if (select == null)
				return o;
			SqlCommand cmd = new SqlCommand(select.Text, select.Connection);
			if (select.Transaction != null)
				cmd.Transaction = select.Transaction;
			SqlDataReader reader = cmd.ExecuteReader();
			try 
			{
				IList list = new ArrayList();
				while(reader.Read()) 
				{
					list.Add(Format(select.RowFormat, reader));
				}
				return list;
			}
			finally
			{
				reader.Close();
			}
		}
		public object Format(string rowFormat, SqlDataReader reader)
		{
			string[] fields = new string[reader.FieldCount];
			for (int i = 0; i < reader.FieldCount; ++i) 
			{
				fields[i] = outermostConverter.ToString(reader[i]);
			}
			if (rowFormat == null)
				return fields;
			return string.Format(rowFormat, fields); 
		}
	}
}
